/**
 * TemperatureModule is the web service client
 * Copyright 2006, The MathWorks, Inc.
*/

import com.mathworks.toolbox.javabuilder.examples.TemperatureCellArray;
import com.mathworks.toolbox.javabuilder.examples.TemperatureStruct;
import com.mathworks.toolbox.javabuilder.*;

import org.apache.axis.AxisFault;
import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.encoding.XMLType;
import org.apache.axis.utils.Options;

import javax.xml.namespace.QName;
import javax.xml.rpc.ParameterMode;
import java.net.URL;


public class TemperatureModuleClient {

    public Object callService (Object tempIn, String tempType) throws Exception {

 	  Service  service = new Service();
	  Call     call    = (Call) service.createCall();

	  //modify the enpoint address as necessary
	  String endpoint =  "http://localhost:8080/axis/services/temperatureModule";
	  call.setTargetEndpointAddress( new java.net.URL(endpoint) );
	  call.setOperationName(new QName("temperatureModule", "convertTemp"));	
	  Object ret=null;	  
	  
	  //use reflection to determine whether to register the bean serializers for
	  //custom classes
	  if(tempIn instanceof TemperatureCellArray)
	  {
	  	
		  QName    qn      = new QName( "urn:TemperatureModule", "TemperatureCellArray" );

		  //specify the serializer and deserializer for custom class TemperatureCellArray
	      call.registerTypeMapping(TemperatureCellArray.class, qn,
	                      new org.apache.axis.encoding.ser.BeanSerializerFactory(TemperatureCellArray.class, qn),        
	                      new org.apache.axis.encoding.ser.BeanDeserializerFactory(TemperatureCellArray.class, qn));
	   
	      call.addParameter("tempIn", qn, ParameterMode.IN);
	      call.addParameter("tempType", XMLType.XSD_STRING, ParameterMode.IN);
		  call.setReturnType(qn); 
	  }
	  
	  if(tempIn instanceof TemperatureStruct)
	  {
	  	
		  //specify the serializer and deserializer for custom class TemperatureStruct
		  QName    qn      = new QName( "urn:TemperatureModule", "TemperatureStruct" );
	      call.registerTypeMapping(TemperatureStruct.class, qn,
	                      new org.apache.axis.encoding.ser.BeanSerializerFactory(TemperatureStruct.class, qn),        
	                      new org.apache.axis.encoding.ser.BeanDeserializerFactory(TemperatureStruct.class, qn));
	   
	      call.addParameter("tempIn", qn, ParameterMode.IN);
	      call.addParameter("tempType", XMLType.XSD_STRING, ParameterMode.IN);
		  call.setReturnType(qn); 
	  }

	  
	  try {
	 	ret = call.invoke( new Object[] {tempIn, tempType} );	 	
      } 
      
      catch(Exception e)
      {
      	System.out.println("ERROR CALLING INVOKE: " + e.getStackTrace());
      	e.printStackTrace();
      	throw e;
      }
      
      return ret;
  }

  public static void main(String args[]) {

	//scalar input specified as double
	double x0 = 36;

   //two-dimensional input specified as double[][]
	double[][] x1 = {{1.0,2.0},{3.0,4.0, 5.0}};
	
   //two-dimensional jagged input specified as custom-defined TemperatureCellArray
	double[][] temperatures1 = {{40},{41,42},{43,44,45}};
	TemperatureCellArray cell = new TemperatureCellArray(temperatures1);			

   //two-dimensional jagged input specified as custom-defined TemperatureStruct
	double[][] temperatures2 = {{1},{3,4},{5,6,7}};
	TemperatureStruct struct = new TemperatureStruct(temperatures2);			

    try 
    {    	
        TemperatureModuleClient tt = new TemperatureModuleClient();
	
		System.out.println("Calling for scalar input");			
        Object ret = tt.callService(x0, "f");
		output(ret);
		
		System.out.println("Calling for matrix input");			
		ret = tt.callService(x1, "f");
		output(ret);
		
		System.out.println("Calling for cell input");
		TemperatureCellArray retCell = (TemperatureCellArray)(tt.callService(cell, "f"));
		output(retCell.getTemperatures());
		
		System.out.println("Calling for struct input");
		TemperatureStruct retStruct = (TemperatureStruct)(tt.callService(struct, "f"));
		
		Object structTemps = retStruct.getTemperatures();
		
		if(structTemps instanceof TemperatureCellArray)
		{
			TemperatureCellArray tempCell = (TemperatureCellArray)(structTemps);
			output(tempCell.getTemperatures());
		}
		
		else
		{
		    output(structTemps);
		}

    }
    catch( Exception e ) 
    {
       e.printStackTrace();
    }
  }
  
  	public static void output(Object out)
			
    {
		if(out instanceof double[][])
		{
			double[][]x = (double[][])out;
			int dim1 = x.length;
			int dim2;
			
			for(int i=0; i<dim1; i++)
			{
			   dim2 = x[i].length;

			   for(int j=0; j<dim2; j++)
			   {
			   	 System.out.println("(" + i + "," + j + ") is " + x[i][j]);
			   }
			}				
		}
	}	
}